package report

import (
	"eta/eta_email_analysis/global"
	"time"
)

type OutsideEmailBaseInfo struct {
	Id              int32     `gorm:"column:id;type:int(10) UNSIGNED;primaryKey;not null;" json:"id"`
	Folder          string    `gorm:"column:folder;type:varchar(255);comment:所属文件夹;" json:"folder"`                           // 所属文件夹
	EmailMessageUid uint32    `gorm:"column:email_message_uid;type:int(10);comment:邮件id;default:0;" json:"email_message_uid"` // 邮件id
	Title           string    `gorm:"column:title;type:varchar(512);comment:邮件主题;" json:"title"`                              // 邮件主题
	FromAddress     string    `gorm:"column:from_address;type:varchar(255);comment:发件人邮箱;" json:"from_address"`               // 发件人邮箱
	From            string    `gorm:"column:from;type:varchar(255);comment:发件人名称;" json:"from"`                               // 发件人名称
	DeliveryTime    time.Time `gorm:"column:delivery_time;type:datetime;comment:收件时间;default:NULL;" json:"delivery_time"`     // 收件时间
	CreateTime      time.Time `gorm:"column:create_time;type:datetime;comment:添加时间;default:NULL;" json:"create_time"`         // 添加时间
}

// TableName get sql table name.获取数据库表名
func (m *OutsideEmailBaseInfo) TableName() string {
	return "outside_email_base_info"
}

// GetByEmailMessageIdAndFolder
// @Description: 根据邮件id和文件夹获取邮件基本信息
// @author: Roc
// @datetime 2024-09-14 17:49:35
// @param emailMessageUid uint32
// @return item *OutsideReport
// @return err error
func (m *OutsideEmailBaseInfo) GetByEmailMessageIdAndFolder(emailMessageUid uint32, folder string) (item *OutsideEmailBaseInfo, err error) {
	err = global.DEFAULT_MYSQL.Where("email_message_uid = ? AND folder = ? ", emailMessageUid, folder).First(&item).Error
	return
}

// Add
// @Description: 新增邮件基础信息
// @author: Roc
// @datetime 2024-09-14 16:23:32
// @param item *OutsideReport
// @return err error
func (m *OutsideEmailBaseInfo) Add(item *OutsideEmailBaseInfo) (err error) {
	err = global.DEFAULT_MYSQL.Create(item).Error
	return
}
